#!/usr/bin/python

"""
Module to test RO manager manifest and aggregation commands

See: http://www.wf4ever-project.org/wiki/display/docs/RO+management+tool
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

log = logging.getLogger(__name__)

if __name__ == "__main__":
    # Add main project directory at start of python path
    sys.path.insert(0, "../..")

import rdflib

from MiscLib import TestUtils

from rocommand import ro
from rocommand import ro_utils
from rocommand import ro_manifest
from rocommand.ro_namespaces import RDF, DCTERMS, RO, AO, ORE

from rocommand.test import TestROSupport
from rocommand.test import TestConfig
from rocommand.test import StdoutContext

from iaeval import ro_minim
from iaeval.ro_minim import MINIM

# Base directory for RO tests in this module
testbase = os.path.dirname(os.path.abspath(__file__))

# Config remote testing
remotehost   = "http://andros.zoo.ox.ac.uk"
remoterobase = "/workspace/wf4ever-ro-catalogue/v0.1/"
remoteroname = "simple-requirements/"
remoteminim  = "simple-requirements-minim.rdf"

# Test suite
class TestMinimAccess(TestROSupport.TestROSupport):
    """
    Test ro annotation commands
    """
    def setUp(self):
        super(TestMinimAccess, self).setUp()
        return

    def tearDown(self):
        super(TestMinimAccess, self).tearDown()
        return

    # Setup local config for Minim tests

    def setupConfig(self):
        return self.setupTestBaseConfig(testbase)

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testSetupConfig(self):
        (configdir, robasedir) = self.setupConfig()
        config = ro_utils.readconfig(configdir)
        self.assertEqual(config["robase"],          os.path.abspath(robasedir))
        self.assertEqual(config["rosrs_uri"],       TestConfig.ro_test_config.ROSRS_URI)
        self.assertEqual(config["rosrs_access_token"],  TestConfig.ro_test_config.ROSRS_ACCESS_TOKEN)
        self.assertEqual(config["username"],        TestConfig.ro_test_config.ROBOXUSERNAME)
        self.assertEqual(config["useremail"],       TestConfig.ro_test_config.ROBOXEMAIL)
        return

    def testMinimRead(self):
        """
        Basic test that Minim test file can be read
        """
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        rouri      = ro_manifest.getRoUri(rodir)
        minimbase  = ro_manifest.getComponentUri(rodir, "Minim-UserRequirements.rdf")
        target     = ro_manifest.getComponentUri(rodir, "docs/UserRequirements-astro.csv")
        constraint = ro_minim.getElementUri(minimbase, "#create/docs/UserRequirements-astro.csv")
        model      = ro_minim.getElementUri(minimbase, "#runnableRequirementRO")
        g = ro_minim.readMinimGraph(minimbase)
        expected_minim = (
            [ (target,     MINIM.hasConstraint, constraint                                          )
            , (constraint, MINIM.forPurpose,    rdflib.Literal('create UserRequirements-astro.csv') )
            , (constraint, MINIM.onResource,    rouri                                               )
            , (constraint, MINIM.toModel,       model                                               )
            , (model,      RDF.type,            MINIM.Model                                         )
            ])
        self.checkTargetGraph(g, expected_minim, msg="Not found in Minim")
        self.deleteTestRo(rodir)
        return

    def testGetConstraints(self):
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        rouri      = ro_manifest.getRoUri(rodir)
        minimbase  = ro_manifest.getComponentUri(rodir, "Minim-UserRequirements.rdf")
        model      = ro_minim.getElementUri(minimbase, "#runnableRequirementRO")
        constraint = ro_minim.getElementUri(minimbase, "#create/docs/UserRequirements-astro.csv")
        # Read Minim as graph, scan constraints and look for expected value
        minimgraph = ro_minim.readMinimGraph(minimbase)
        constraints = ro_minim.getConstraints(minimgraph)
        expected_found = False
        for c in constraints:
            if ( c['target']   == ro_manifest.getComponentUri(rodir, "docs/UserRequirements-astro.csv") and
                 c['purpose']  == rdflib.Literal("create UserRequirements-astro.csv")                   and
                 c['resource'] == rouri                                                               and
                 c['model']    == model                                                               and
                 c['uri']      == constraint ) :
                expected_found = True
                break
        self.assertTrue(expected_found, "Expected constraint not found in minim")
        return

    def testGetConstraint(self):
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        rouri      = ro_manifest.getRoUri(rodir)
        minimbase  = ro_manifest.getComponentUri(rodir, "Minim-UserRequirements.rdf")
        model      = ro_minim.getElementUri(minimbase, "#runnableRequirementRO")
        constraint = ro_minim.getElementUri(minimbase, "#create/docs/UserRequirements-astro.csv")
        minimgraph = ro_minim.readMinimGraph(minimbase)
        c = ro_minim.getConstraint(minimgraph, rodir,
            "docs/UserRequirements-astro.csv",
            r"create.*UserRequirements-astro\.csv")
        self.assertEquals(c['target'],   ro_manifest.getComponentUri(rodir, "docs/UserRequirements-astro.csv"))
        self.assertEquals(c['purpose'],  rdflib.Literal("create UserRequirements-astro.csv"))
        self.assertEquals(c['resource'], rouri)
        self.assertEquals(c['model'],    model)
        self.assertEquals(c['uri'],      constraint)
        return


    def testGetModels(self):
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        rouri      = ro_manifest.getRoUri(rodir)
        minimbase  = ro_manifest.getComponentUri(rodir, "Minim-UserRequirements.rdf")
        model      = ro_minim.getElementUri(minimbase, "#runnableRequirementRO")
        minimgraph = ro_minim.readMinimGraph(minimbase)
        models     = ro_minim.getModels(minimgraph)
        expected_found = False
        for m in models:
            if ( m['label']  == rdflib.Literal("Runnable Requirements RO") and
                 m['uri']    == model ) :
                expected_found = True
                break
        self.assertTrue(expected_found, "Expected model not found in minim")
        return
    
    def testGetModel(self):
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        rouri      = ro_manifest.getRoUri(rodir)
        minimbase  = ro_manifest.getComponentUri(rodir, "Minim-UserRequirements.rdf")
        model      = ro_minim.getElementUri(minimbase, "#runnableRequirementRO")
        minimgraph = ro_minim.readMinimGraph(minimbase)
        m = ro_minim.getModel(minimgraph, model)
        self.assertEquals(m['label'], rdflib.Literal("Runnable Requirements RO"))
        self.assertEquals(m['uri'],   model)
        return

    def testGetRequirements(self):
        def compare_reqs(req_expect, req_found):
            for k in req_expect:
                if not k in req_found:
                    log.debug("- not found: %s"%(k))
                    return False
                elif isinstance(req_expect[k], dict) and isinstance(req_found[k], dict):
                    if not compare_reqs(req_expect[k], req_found[k]): return False
                elif req_found[k] != req_expect[k]:
                    log.debug("- not found: %s: %s != %s "%(k,req_expect[k],req_found[k]))
                    return False
            return True
        self.setupConfig()
        rodir        = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        rouri        = ro_manifest.getRoUri(rodir)
        minimbase    = ro_manifest.getComponentUri(rodir, "Minim-UserRequirements.rdf")
        model        = ro_minim.getElementUri(minimbase, "#runnableRequirementRO")
        minimgraph   = ro_minim.readMinimGraph(minimbase)
        requirements = ro_minim.getRequirements(minimgraph, model)
        expected_found = False
        r1 = (
            { 'level': "MUST"
            , 'label': rdflib.Literal("Python 2.7")
            , 'softwarerule':
              { 'command':  rdflib.Literal("python -V")
              , 'response': rdflib.Literal("Python 2.7")
              , 'derives':  ro_minim.getElementUri(minimbase, "#environment-software/python-27")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#environment-software/python-27") 
            })
        r2 = (
            { 'level': "MUST"
            , 'label': rdflib.Literal("aggregates data/UserRequirements-astro.ods")
            , 'datarule':
              { 'aggregates': ro_manifest.getComponentUri(rodir, "data/UserRequirements-astro.ods")
              , 'derives':    ro_minim.getElementUri(minimbase, "#isPresent/data/UserRequirements-astro.ods")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/data/UserRequirements-astro.ods") 
            })
        r3 = (
            { 'level': "SHOULD"
            , 'label': rdflib.Literal("aggregates docs/reqs.css")
            , 'datarule':
              { 'aggregates': ro_manifest.getComponentUri(rodir, "docs/reqs.css") 
              , 'derives':    ro_minim.getElementUri(minimbase, "#isPresent/docs/reqs.css")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/docs/reqs.css") 
            })
        r1_found = r2_found = r3_found = False
        for r in requirements:
            log.debug("requirement: %s"%(repr(r)))
            if compare_reqs(r1, r): r1_found = True
            if compare_reqs(r2, r): r2_found = True
            if compare_reqs(r3, r): r3_found = True
        self.assertTrue(r1_found, "Expected requirement(1) not found in minim")
        self.assertTrue(r2_found, "Expected requirement(2) not found in minim")
        self.assertTrue(r3_found, "Expected requirement(3) not found in minim")
        return

    # Remote access tests

    def testMinimReadRemote(self):
        """
        Basic test that remote Minim test file can be read
        """
        rouri      = rdflib.URIRef(remotehost+remoterobase+remoteroname)
        minimbase  = rdflib.URIRef(remotehost+remoterobase+remoteroname+remoteminim)
        constraint = ro_minim.getElementUri(minimbase, "#runnable_RO")
        model      = ro_minim.getElementUri(minimbase, "#runnable_RO_model")
        target     = rouri
        g = ro_minim.readMinimGraph(minimbase)
        expected_minim = (
            [ (target,     MINIM.hasConstraint, constraint                 )
            , (constraint, MINIM.forPurpose,    rdflib.Literal('Runnable') )
            , (constraint, MINIM.onResource,    rouri                      )
            , (constraint, MINIM.toModel,       model                      )
            , (model,      RDF.type,            MINIM.Model                )
            ])
        self.checkTargetGraph(g, expected_minim, msg="Not found in Minim")
        return

    def testGetConstraintsRemote(self):
        rouri      = rdflib.URIRef(remotehost+remoterobase+remoteroname)
        minimbase  = rdflib.URIRef(remotehost+remoterobase+remoteroname+remoteminim)
        constraint = ro_minim.getElementUri(minimbase, "#runnable_RO")
        model      = ro_minim.getElementUri(minimbase, "#runnable_RO_model")
        # Read Minim as graph, scan constraints and look for expected value
        minimgraph = ro_minim.readMinimGraph(minimbase)
        constraints = ro_minim.getConstraints(minimgraph)
        expected_found = False
        for c in constraints:
            if ( c['target']   == rouri                      and
                 c['purpose']  == rdflib.Literal("Runnable") and
                 c['resource'] == rouri                      and
                 c['model']    == model                      and
                 c['uri']      == constraint ) :
                expected_found = True
                break
        self.assertTrue(expected_found, "Expected constraint not found in minim")
        return

    def testGetConstraintRemote(self):
        rouri      = rdflib.URIRef(remotehost+remoterobase+remoteroname)
        minimbase  = rdflib.URIRef(remotehost+remoterobase+remoteroname+remoteminim)
        constraint = ro_minim.getElementUri(minimbase, "#runnable_RO")
        model      = ro_minim.getElementUri(minimbase, "#runnable_RO_model")
        minimgraph = ro_minim.readMinimGraph(minimbase)
        c = ro_minim.getConstraint(minimgraph, rouri,
            ".",
            "Runnable")
        self.assertEquals(c['target'],   rouri)
        self.assertEquals(c['purpose'],  rdflib.Literal("Runnable"))
        self.assertEquals(c['resource'], rouri)
        self.assertEquals(c['model'],    model)
        self.assertEquals(c['uri'],      constraint)
        return

    def testGetModelsRemote(self):
        rouri      = rdflib.URIRef(remotehost+remoterobase+remoteroname)
        minimbase  = rdflib.URIRef(remotehost+remoterobase+remoteroname+remoteminim)
        constraint = ro_minim.getElementUri(minimbase, "#runnable_RO")
        model      = ro_minim.getElementUri(minimbase, "#runnable_RO_model")
        minimgraph = ro_minim.readMinimGraph(minimbase)
        models     = ro_minim.getModels(minimgraph)
        expected_found = False
        for m in models:
            if ( m['label']  == rdflib.Literal("Runnable RO") and
                 m['uri']    == model ) :
                expected_found = True
                break
        self.assertTrue(expected_found, "Expected model not found in minim")
        return
    
    def testGetModelRemote(self):
        rouri      = rdflib.URIRef(remotehost+remoterobase+remoteroname)
        minimbase  = rdflib.URIRef(remotehost+remoterobase+remoteroname+remoteminim)
        constraint = ro_minim.getElementUri(minimbase, "#runnable_RO")
        model      = ro_minim.getElementUri(minimbase, "#runnable_RO_model")
        minimgraph = ro_minim.readMinimGraph(minimbase)
        m = ro_minim.getModel(minimgraph, model)
        self.assertEquals(m['label'], rdflib.Literal("Runnable RO"))
        self.assertEquals(m['uri'],   model)
        return

    def testGetRequirementsRemote(self):
        def compare_reqs(req_expect, req_found):
            for k in req_expect:
                if not k in req_found:
                    log.debug("- not found: %s"%(k))
                    return False
                elif isinstance(req_expect[k], dict) and isinstance(req_found[k], dict):
                    if not compare_reqs(req_expect[k], req_found[k]): return False
                elif req_found[k] != req_expect[k]:
                    log.debug("- not found: %s: %s != %s "%(k,req_expect[k],req_found[k]))
                    return False
            return True
        rouri      = rdflib.URIRef(remotehost+remoterobase+remoteroname)
        minimbase  = rdflib.URIRef(remotehost+remoterobase+remoteroname+remoteminim)
        constraint = ro_minim.getElementUri(minimbase, "#runnable_RO")
        model      = ro_minim.getElementUri(minimbase, "#runnable_RO_model")
        minimgraph = ro_minim.readMinimGraph(minimbase)
        requirements = ro_minim.getRequirements(minimgraph, model)
        expected_found = False
        r1 = (
            { 'level': "MUST"
            , 'label': None
            , 'softwarerule':
              { 'command':  rdflib.Literal("python -V")
              , 'response': rdflib.Literal("Python 2.7")
              , 'derives':  ro_minim.getElementUri(minimbase, "#environment-software/Python 2.7")
              , 'show': rdflib.term.Literal('python -V command %(command)s returns %(response)s')
              , 'showfail': None
              , 'showpass': None
              }
            , 'model': model
            , 'uri': ro_minim.getElementUri(minimbase, "#environment-software/python-27") 
            })
        r2 = (
            { 'level': 'MUST'
            , 'label': None
            , 'contentmatchrule':
              { 'exists':   rdflib.term.Literal('\n          ?wf rdf:type wfdesc:Workflow .\n        ')
              , 'show':     None
              , 'template': None
              , 'forall':   None
              , 'derives':  ro_minim.getElementUri(minimbase, "#isPresent/workflow-instance")
              , 'showfail': rdflib.term.Literal('No workflow instance or template found')
              , 'showpass': rdflib.term.Literal('Workflow instance or template found')
              }
            , 'model': model
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/workflow-instance")
            })
        r1_found = r2_found = False
        for r in requirements:
            log.debug("requirement: %s"%(repr(r)))
            if compare_reqs(r1, r): r1_found = True
            if compare_reqs(r2, r): r2_found = True
        self.assertTrue(r1_found, "Expected requirement(1) not found in minim")
        self.assertTrue(r2_found, "Expected requirement(2) not found in minim")
        return

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            , "testSetupConfig"
            , "testMinimRead"
            , "testGetConstraints"
            , "testGetConstraint"
            , "testGetModels"
            , "testGetModel"
            , "testGetRequirements"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            , "testMinimReadRemote"
            , "testGetConstraintsRemote"
            , "testGetConstraintRemote"
            , "testGetModelsRemote"
            , "testGetModelRemote"
            , "testGetRequirementsRemote"
            ]
        }
    return TestUtils.getTestSuite(TestMinimAccess, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestMinimAccess.log", getTestSuite, sys.argv)

# End.



# ro_annotation.py

"""
Research Object annotation read, write, decode functions
"""

import sys
import os
import os.path
import datetime
import logging
import re

log = logging.getLogger(__name__)

import rdflib
#from rdflib.namespace import RDF, RDFS
#from rdflib import URIRef, Namespace, BNode
#from rdflib import Literal

import ro_settings
import ro_manifest
from ro_namespaces import RDF, RDFS, RO, AO, ORE, DCTERMS, ROTERMS

#   Default list of annotation types
annotationTypes = (
    [
      { "name": "type", "prefix": "dcterms", "localName": "type", "type": "string"
      , "baseUri": DCTERMS.baseUri, "fullUri": DCTERMS.type
      , "label": "Type"
      , "description": "Word or brief phrase describing type of Research Object component"
      }
    , { "name": "keywords", "prefix": "dcterms", "localName": "subject", "type": "termlist"
      , "baseUri": DCTERMS.baseUri, "fullUri": DCTERMS.subject
      , "label": "Keywords"
      , "description": "List of key words or phrases associated with a Research Object component"
      }
    , { "name": "description", "prefix": "dcterms", "localName": "description", "type": "text"
      , "baseUri": DCTERMS.baseUri, "fullUri": DCTERMS.description
      , "label": "Description"
      , "description": "Extended description of Research Object component"
      }
    , { "name": "format", "prefix": "dcterms", "localName": "format", "type": "string"
      , "baseUri": DCTERMS.baseUri, "fullUri": DCTERMS.format
      , "label": "Data format"
      , "description": "String indicating the data format of a Research Object component"
      }
    , { "name": "note", "prefix": "dcterms", "localName": "note", "type": "text"
      , "baseUri": ROTERMS.baseUri, "fullUri": ROTERMS.note
      , "label": "Note"
      , "description": "String indicating some information about a Research Object component"
      }
    , { "name": "title", "prefix": "dcterms", "localName": "title", "type": "string"
      , "baseUri": DCTERMS.baseUri, "fullUri": DCTERMS.title
      , "label": "Title"
      , "description": "Title of Research Object component"
      }
    , { "name": "created", "prefix": "dcterms", "localName": "created", "type": "datetime"
      , "baseUri": DCTERMS.baseUri, "fullUri": DCTERMS.created
      , "label": "Creation time"
      , "description": "Date and time that Research Object component was created"
      }
    , { "name": "rdf:type", "prefix": "rdf", "localName": "type", "type": "resource"
      , "baseUri": RDF.baseUri, "fullUri": RDF.type
      , "label": "RDF type"
      , "description": "RDF type of the annotated object"
      }
    , { "name": "rdfs:seeAlso", "prefix": "rdfs", "localName": "seeAlso", "type": "resource"
      , "baseUri": RDFS.baseUri, "fullUri": RDFS.seeAlso
      , "label": "See also"
      , "description": "Related resource with further information"
      }
    ])

# Default list of annotation prefixes
annotationPrefixes = (
    { "rdf":       "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    , "rdfs":      "http://www.w3.org/2000/01/rdf-schema#"
    , "owl":       "http://www.w3.org/2002/07/owl#"
    , "xsd":       "http://www.w3.org/2001/XMLSchema#"
    , "rdfg":      "http://www.w3.org/2004/03/trix/trix-1/"
    , "skos":      "http://www.w3.org/2004/02/skos/core#"
    , "foaf":      "http://xmlns.com/foaf/0.1/"
    , "dc":        "http://purl.org/dc/elements/1.1/"
    , "dcterms":   "http://purl.org/dc/terms/"
    , "ore":       "http://www.openarchives.org/ore/terms/"
    , "ao":        "http://purl.org/ao/"
    , "ro":        "http://purl.org/wf4ever/ro#"
    , "roterms":   "http://purl.org/wf4ever/roterms#"
    , "roevo":     "http://purl.org/wf4ever/roevo#"
    , "wfdesc":    "http://purl.org/wf4ever/wfdesc#"
    , "wfprov":    "http://purl.org/wf4ever/wfprov#"
    })

# Annotation support functions
def getAnnotationByName(ro_config, aname, defaultType="string"):
    """
    Given an attribute name from the command line, returns
    attribute predicate and type URIs as a URIRef node and attribute value type
    """
    #@@TODO: deal properly with annotation types: return URIRef for type instead of name string
    # Default interpret atrtribute name as URI
    predicate = aname
    valtype   = defaultType
    asplit = aname.split(":")
    for atype in ro_config["annotationTypes"]:
        # Try to interpret attribute name as predefined name
        if atype["name"] == aname:
            predicate = atype["fullUri"]
            valtype   = atype["type"]
            break
    else:
        if len(asplit) == 2:
            # Try to interpret name as CURIE
            for apref in ro_config["annotationPrefixes"]:
                if asplit[0] == apref:
                    predicate = ro_config["annotationPrefixes"][apref]+asplit[1]
    predicate = rdflib.URIRef(predicate)
    return (predicate, valtype)

def getAnnotationByUri(ro_config, auri, defaultType="string"):
    """
    Given an attribute URI from the manifest graph, returns an
    attribute name and type tuple for displaying an attribute
    """
    # Look for predefined name
    for atype in ro_config["annotationTypes"]:
        if str(atype["fullUri"]) == str(auri):
            return (atype["name"], atype["type"])
    # Look for CURIE match
    for (pref,puri) in ro_config["annotationPrefixes"].iteritems():
        if auri.startswith(puri):
            return (pref+":"+auri[len(puri):], defaultType)
    # return full URI in angle brackets
    return ("<"+str(auri)+">", defaultType)

def getAnnotationNameByUri(ro_config, uri):
    """
    Given an attribute URI from the manifest graph, returns an
    attribute name for displaying an attribute
    """
    return getAnnotationByUri(ro_config, uri)[0]

def makeAnnotationFilename(rodir, afile):
    #log.debug("makeAnnotationFilename: %s, %s"%(rodir, afile))
    return os.path.join(os.path.abspath(rodir), ro_settings.MANIFEST_DIR+"/", afile)

def makeComponentFilename(rodir, rofile):
    log.debug("makeComponentFilename: %s, %s"%(rodir, rofile))
    return os.path.join(rodir, rofile)

def readAnnotationBody(rodir, annotationfile):
    """
    Read annotation body from indicated file, return RDF Graph of annotation values.
    """
    annotationfilename = makeComponentFilename(rodir, annotationfile)
    if not os.path.exists(annotationfilename): return None
    annotationformat   = "xml"
    # Look at file extension to figure format
    if re.search("\.(ttl|n3)$", annotationfile): annotationformat="n3"
    rdfGraph = rdflib.Graph()
    rdfGraph.parse(annotationfilename, format=annotationformat)
    return rdfGraph

def createAnnotationGraphBody(ro_config, ro_dir, rofile, anngraph):
    """
    Create a new annotation body for a single resource in a research object, based
    on a supplied graph value.

    Existing annotations for the same resource are not touched; if an annotation is being
    added or replaced, it is the calling program'sresponsibility to update the manifest to
    reference the active annotations.  A new name is allocated for the created annotation,
    graph which is returned as the result of this function.

    ro_config   is the research object manager configuration, supplied as a dictionary
    ro_dir      is the research object root directory
    rofile      is the name of the Research Object component to be annotated, possibly
                relative to the RO root directory.
    anngraph    is an annotation graph that is to be saved.

    Returns the name of the annotation body created relative to the RO
    manifest and metadata directory.
    """
    # Determine name for annotation body
    log.debug("createAnnotationGraphBody: %s, %s"%(ro_dir, rofile))
    annotation_filename = None
    name_index = 0
    name_suffix = os.path.basename(rofile)
    if name_suffix in [".",""]:
        name_suffix = os.path.basename(os.path.normpath(ro_dir))
    today = datetime.date.today()
    while annotation_filename == None:
        name_index += 1
        name = ("Ann-%04d%02d%02d-%04d-%s.rdf"%
                (today.year, today.month, today.day, name_index, name_suffix))
        if not os.path.exists(makeAnnotationFilename(ro_dir, name)):
            annotation_filename = name
    # Create annotation body file
    log.debug("createAnnotationGraphBody: %s"%(annotation_filename))
    anngraph.serialize(destination=makeAnnotationFilename(ro_dir, annotation_filename),
        format='xml', base=ro_manifest.getRoUri(ro_dir), xml_base="..")
    return annotation_filename

def createAnnotationBody(ro_config, ro_dir, rofile, attrdict, defaultType="string"):
    """
    Create a new annotation body for a single resource in a research object.

    Existing annotations for the same resource are not touched; if an annotation is being
    added or replaced, it is the calling program'sresponsibility to update the manifest to
    reference the active annotations.  A new name is allocated for the created annotation,
    which is returned as the result of this function.

    ro_config   is the research object manager configuration, supplied as a dictionary
    ro_dir      is the research object root directory
    rofile      is the name of the Research Object component to be annotated, possibly
                relative to the RO root directory.
    attrdict    is a dictionary of attributes to be saved inthe annotation body.
                Dictionary keys are attribute names that can be resolved via getAnnotationByName.

    Returns the name of the annotation body created relative to the RO
    manifest and metadata directory.
    """
    # Assemble data for annotation
    anngraph = rdflib.Graph()
    s = ro_manifest.getComponentUri(ro_dir, rofile)
    for k in attrdict:
        (p,t) = getAnnotationByName(ro_config, k, defaultType)
        anngraph.add((s, p, makeAnnotationValue(attrdict[k],t)))
    # Write graph and return filename
    return createAnnotationGraphBody(ro_config, ro_dir, rofile, anngraph)

def _addAnnotationBodyToRoGraph(ro_graph, ro_dir, rofile, annfile):
    """
    Add a new annotation body to an RO graph

    ro_graph    graph to which annotation is added
    ro_dir      is the research object directory
    rofile      is the research object file being annotated
    annfile     is the base file name of the annotation body to be added
    """
    # <ore:aggregates>
    #   <ro:AggregatedAnnotation>
    #     <ro:annotatesAggregatedResource rdf:resource="data/UserRequirements-astro.ods" />
    #     <ao:body rdf:resource=".ro/(annotation).rdf" />
    #   </ro:AggregatedAnnotation>
    # </ore:aggregates>
    ann = rdflib.BNode()
    ro_graph.add((ann, RDF.type, RO.AggregatedAnnotation))
    ro_graph.add((ann, RO.annotatesAggregatedResource, ro_manifest.getComponentUri(ro_dir, rofile)))
    ro_graph.add((ann, AO.body, ro_manifest.getComponentUri(ro_dir, ro_settings.MANIFEST_DIR+"/"+annfile)))
    ro_graph.add((ro_manifest.getComponentUri(ro_dir, "."), ORE.aggregates, ann))
    return

def _removeAnnotationBodyFromRoGraph(ro_graph, annbody):
    """
    Remove references to an annotation body from an RO graph

    ro_graph    graph from which annotation is removed
    annbody     is the the annotation body node to be removed
    """
    ro_graph.remove((annbody, None,           None   ))
    ro_graph.remove((None,    ORE.aggregates, annbody))
    return

def _addSimpleAnnotation(ro_config, ro_dir, rofile, attrname, attrvalue):
    """
    Add a simple annotation to a file in a research object.

    ro_config   is the research object manager configuration, supplied as a dictionary
    ro_dir      is the research object root directory
    rofile      names the file or resource to be annotated, possibly relative to the RO.
    attrname    names the attribute in a form recognized by getAnnotationByName
    attrvalue   is a value to be associated with the attribute
    """
    annfile = createAnnotationBody(ro_config, ro_dir, rofile, { attrname: attrvalue} )
    ro_graph = ro_manifest.readManifestGraph(ro_dir)
    _addAnnotationBodyToRoGraph(ro_graph, ro_dir, rofile, annfile)
    ro_manifest.writeManifestGraph(ro_dir, ro_graph)
    return

def _removeSimpleAnnotation(ro_config, ro_dir, rofile, attrname, attrvalue):
    """
    Remove a simple annotation or multiple matching annotations a research object.

    ro_config   is the research object manager configuration, supplied as a dictionary
    ro_dir      is the research object root directory
    rofile      names the annotated file or resource, possibly relative to the RO.
    attrname    names the attribute in a form recognized by getAnnotationByName
    attrvalue   is the attribute value to be deleted, or Nomne to delete all vaues
    """
    log.debug("removeSimpleAnnotation: ro_dir %s, rofile %s, attrname %s, attrvalue %s"%
              (ro_dir, rofile, attrname, attrvalue))
    # Enumerate annotations
    # For each:
    #     if annotation is only one in graph then:
    #         remove aggregated annotation
    #     else:
    #         create new annotation graph witj annotation removed
    #         update aggregated annotation
    ro_graph    = ro_manifest.readManifestGraph(ro_dir)
    subject     = ro_manifest.getComponentUri(ro_dir, rofile)
    (predicate,valtype) = getAnnotationByName(ro_config, attrname)
    val         = attrvalue and makeAnnotationValue(attrvalue, valtype)
    #@@TODO refactor common code with getRoAnnotations, etc.
    add_annotations = []
    remove_annotations = []
    for ann_node in ro_graph.subjects(predicate=RO.annotatesAggregatedResource, object=subject):
        ann_uri   = ro_graph.value(subject=ann_node, predicate=AO.body)
        ann_graph = readAnnotationBody(ro_dir, ro_manifest.getComponentUriRel(ro_dir, ann_uri))
        if (subject, predicate, val) in ann_graph:
            ann_graph.remove((subject, predicate, val))
            if (subject, None, None) in ann_graph:
                # Triples remain in annotation body: write new body and update RO graph
                ann_name = createAnnotationGraphBody(ro_config, ro_dir, rofile, ann_graph)
                remove_annotations.append(ann_node)
                add_annotations.append(ann_name)
            else:
                # Remove annotation from RO graph
                remove_annotations.append(ann_node)
    # Update RO graph if needed
    if add_annotations or remove_annotations:
        for a in remove_annotations:
            _removeAnnotationBodyFromRoGraph(ro_graph, a)
        for a in add_annotations:
            _addAnnotationBodyToRoGraph(ro_graph, ro_dir, rofile, a)
        ro_manifest.writeManifestGraph(ro_dir, ro_graph)
    return

def _replaceSimpleAnnotation(ro_config, ro_dir, rofile, attrname, attrvalue):
    """
    Replace a simple annotation in a research object.

    ro_config   is the research object manager configuration, supplied as a dictionary
    ro_dir      is the research object root directory
    rofile      names the file or resource to be annotated, possibly relative to the RO.
    attrname    names the attribute in a form recognized by getAnnotationByName
    attrvalue   is a new value to be associated with the attribute
    """
    ro_graph = ro_manifest.readManifestGraph(ro_dir)
    subject  = ro_manifest.getComponentUri(ro_dir, rofile)
    (predicate,valtype) = getAnnotationByName(ro_config, attrname)
    log.debug("Replace annotation: subject %s, predicate %s, value %s"%(repr(subject), repr(predicate), repr(attrvalue)))
    ro_graph.remove((subject, predicate, None))
    ro_graph.add((subject, predicate, makeAnnotationValue(attrvalue, valtype)))
    ro_manifest.writeManifestGraph(ro_dir, ro_graph)
    return

def _getAnnotationValues(ro_config, ro_dir, rofile, attrname):
    """
    Returns iterator over annotation values for given subject and attribute
    """
    log.debug("getAnnotationValues: ro_dir %s, rofile %s, attrname %s"%(ro_dir, rofile, attrname))
    ro_graph    = ro_manifest.readManifestGraph(ro_dir)
    subject     = ro_manifest.getComponentUri(ro_dir, rofile)
    (predicate,valtype) = getAnnotationByName(ro_config, attrname)
    #@@TODO refactor common code with getRoAnnotations, etc.
    for ann_node in ro_graph.subjects(predicate=RO.annotatesAggregatedResource, object=subject):
        ann_uri   = ro_graph.value(subject=ann_node, predicate=AO.body)
        ann_graph = readAnnotationBody(ro_dir, ro_manifest.getComponentUriRel(ro_dir, ann_uri))
        for v in ann_graph.objects(subject=subject, predicate=predicate):
            #log.debug("Triple: %s %s %s"%(subject,p,v))
            yield v
    return

def _getRoAnnotations(ro_dir):
    """
    Returns iterator over annotations applied to the RO as an entity.

    Each value returned by the iterator is a (subject,predicate,object) triple.
    """
    ro_graph = ro_manifest.readManifestGraph(ro_dir)
    subject  = ro_manifest.getRoUri(ro_dir)
    log.debug("getRoAnnotations %s"%str(subject))
    for ann_node in ro_graph.subjects(predicate=RO.annotatesAggregatedResource, object=subject):
        ann_uri   = ro_graph.value(subject=ann_node, predicate=AO.body)
        ann_graph = readAnnotationBody(ro_dir, ro_manifest.getComponentUriRel(ro_dir, ann_uri))
        if ann_graph:
            for (p, v) in ann_graph.predicate_objects(subject=subject):
                #log.debug("Triple: %s %s %s"%(subject,p,v))
                yield (subject, p, v)
    return

def _getFileAnnotations(ro_dir, rofile):
    """
    Returns iterator over annotations applied to a specified component in the RO

    Each value returned by the iterator is a (subject,predicate,object) triple.
    """
    log.debug("getFileAnnotations: ro_dir %s, rofile %s"%(ro_dir, rofile))
    ro_graph    = ro_manifest.readManifestGraph(ro_dir)
    subject     = ro_manifest.getComponentUri(ro_dir, rofile)
    log.debug("getFileAnnotations: %s"%str(subject))
    #@@TODO refactor common code with getRoAnnotations, etc.
    for ann_node in ro_graph.subjects(predicate=RO.annotatesAggregatedResource, object=subject):
        ann_uri   = ro_graph.value(subject=ann_node, predicate=AO.body)
        ann_graph = readAnnotationBody(ro_dir, ro_manifest.getComponentUriRel(ro_dir, ann_uri))
        if ann_graph:
            for (p, v) in ann_graph.predicate_objects(subject=subject):
                #log.debug("Triple: %s %s %s"%(subject,p,v))
                yield (subject, p, v)
    return

def getAllAnnotations(ro_dir):
    """
    Returns iterator over all annotations associated with the RO

    Each value returned by the iterator is a (subject,predicate,object) triple.
    """
    log.debug("getAllAnnotations %s"%str(ro_dir))
    ro_graph    = ro_manifest.readManifestGraph(ro_dir)
    #@@TODO refactor common code with getRoAnnotations, etc.
    for (ann_node, subject) in ro_graph.subject_objects(predicate=RO.annotatesAggregatedResource):
        ann_uri   = ro_graph.value(subject=ann_node, predicate=AO.body)
        ann_graph = readAnnotationBody(ro_dir, ro_manifest.getComponentUriRel(ro_dir, ann_uri))
        if ann_graph == None:
            log.debug("No annotation graph: ann_uri: "+str(ann_uri))
        else:
            for (p, v) in ann_graph.predicate_objects(subject=subject):
                #log.debug("Triple: %s %s %s"%(subject,p,v))
                yield (subject, p, v)
    return

def makeAnnotationValue(aval, atype):
    """
    atype is one of "string", "resurce", ...

    Returns a graph node for the supplied type and value
    """
    #@@TODO: construct appropriately typed RDF literals
    if atype == "string":
        return rdflib.Literal(aval)
    if atype == "text":
        return rdflib.Literal(aval)
    if atype == "datetime":
        return rdflib.Literal(aval)
    if atype == "resource":
        return rdflib.URIRef(aval)
    return rdflib.Literal(aval)

def formatAnnotationValue(aval, atype):
    """
    atype is one of "string", "resurce", ...
    """
    #@@TODO: deal with appropriately typed RDF literals
    if atype == "string":
        return '"' + str(aval).replace('"', '\\"') + '"'
    if atype == "text":
        # multiline
        return '"""' + str(aval) + '"""'
    if atype == "datetime":
        return '"' + str(aval) + '"'
    if atype == "resource":
        return '<' + str(aval) + '>'
    return str(aval)

def showAnnotations(ro_config, ro_dir, annotations, outstr):
    sname_prev = None
    for (asubj,apred,aval) in annotations:
        #log.debug("Annotations: asubj %s, apred %s, aval %s"%
        #          (repr(asubj), repr(apred), repr(aval)))
        if apred != ORE.aggregates:
            (aname, atype) = getAnnotationByUri(ro_config, apred)
            sname = ro_manifest.getComponentUriRel(ro_dir, str(asubj))
            log.debug("Annotations: sname %s, aname %s"%(sname, aname))
            if sname == "":
                sname = ro_manifest.getRoUri(ro_dir)
            if sname != sname_prev:
                print "\n<"+sname+">"
                sname_prev = sname
            outstr.write("  %s %s\n"%(aname, formatAnnotationValue(aval, atype)))
    return

# End.



#!/usr/bin/python

"""
Module to test basic RO manager commands

See: http://www.wf4ever-project.org/wiki/display/docs/RO+management+tool
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "../..")
    sys.path.insert(0, "..")

from MiscLib import TestUtils

from rocommand import ro, ro_utils

from TestConfig import ro_test_config
from StdoutContext import SwitchStdout

import TestROSupport

# Base directory for RO tests in this module
testbase = os.path.dirname(__file__)

class TestBasicCommands(TestROSupport.TestROSupport):
    """
    Test basic ro commands
    """
    def setUp(self):
        super(TestBasicCommands, self).setUp()
        return

    def tearDown(self):
        super(TestBasicCommands, self).tearDown()
        return

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testHelpVersion(self):
        status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, ["ro", "--version"])
        self.assertEqual(status, 0)
        return

    def testHelpOptions(self):
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, ["ro", "--help"])
            self.assertEqual(status, 0)
        self.assertEqual(self.outstr.getvalue().count("help"), 2)
        return

    def testHelpCommands(self):
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, ["ro", "help"])
            self.assertEqual(status, 0)
        self.assertEqual(self.outstr.getvalue().count("help"), 2)
        return

    def testInvalidCommand(self):
        with SwitchStdout(self.outstr):
            status = ro.runCommand(
                ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, 
                ["ro", "nosuchcommand"])
            self.assertEqual(status, 2)
        self.assertEqual(self.outstr.getvalue().count("Unrecognized"), 1)
        self.assertEqual(self.outstr.getvalue().count("nosuchcommand"), 1)
        return

    def testConfig(self):
        """
        $ ro config \
          -r http://calatola.man.poznan.pl/robox/dropbox_accounts/1/ro_containers/2 \
          -p d41d8cd98f00b204e9800998ecf8427e \
          -b /usr/workspace/Dropbox/myROs \
          -n "Graham Klyne" \
          -e gk@example.org
        """
        ro_utils.resetconfig(ro_test_config.CONFIGDIR)
        config = ro_utils.readconfig(ro_test_config.CONFIGDIR)
        self.assertEqual(config["robase"],    None)
        self.assertEqual(config["rosrs_uri"],  None)
        self.assertEqual(config["rosrs_access_token"],  None)
        self.assertEqual(config["username"],  None)
        self.assertEqual(config["useremail"], None)
        args = [
            "ro", "config",
            "-b", ro_test_config.ROBASEDIR,
            "-r", ro_test_config.ROSRS_URI,
            "-t", ro_test_config.ROSRS_ACCESS_TOKEN,
            "-n", ro_test_config.ROBOXUSERNAME,
            "-e", ro_test_config.ROBOXEMAIL
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
            assert status == 0
        self.assertEqual(self.outstr.getvalue().count("ro config"), 0)
        config = ro_utils.readconfig(ro_test_config.CONFIGDIR)
        self.assertEqual(config["robase"],          os.path.abspath(ro_test_config.ROBASEDIR))
        self.assertEqual(config["rosrs_uri"],       ro_test_config.ROSRS_URI)
        self.assertEqual(config["rosrs_access_token"],  ro_test_config.ROSRS_ACCESS_TOKEN)
        self.assertEqual(config["username"],        ro_test_config.ROBOXUSERNAME)
        self.assertEqual(config["useremail"],       ro_test_config.ROBOXEMAIL)
        return

    def testConfigVerbose(self):
        """
        $ ro config -v \
          -r http://calatola.man.poznan.pl/robox/dropbox_accounts/1/ro_containers/2 \
          -p d41d8cd98f00b204e9800998ecf8427e \
          -b /usr/workspace/Dropbox/myROs \
          -n "Graham Klyne" \
          -e gk@example.org
        """
        ro_utils.resetconfig(ro_test_config.CONFIGDIR)
        config = ro_utils.readconfig(ro_test_config.CONFIGDIR)
        self.assertEqual(config["robase"],    None)
        self.assertEqual(config["rosrs_uri"],  None)
        self.assertEqual(config["rosrs_access_token"],  None)
        self.assertEqual(config["username"],  None)
        self.assertEqual(config["useremail"], None)
        args = [
            "ro", "config", "-v",
            "-b", ro_test_config.ROBASEDIR,
            "-r", ro_test_config.ROSRS_URI,
            "-t", ro_test_config.ROSRS_ACCESS_TOKEN,
            "-n", ro_test_config.ROBOXUSERNAME,
            "-e", ro_test_config.ROBOXEMAIL
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        self.assertEqual(self.outstr.getvalue().count("ro configuration written"), 1)
        config = ro_utils.readconfig(ro_test_config.CONFIGDIR)
        self.assertEqual(config["robase"],    os.path.abspath(ro_test_config.ROBASEDIR))
        self.assertEqual(config["rosrs_uri"],       ro_test_config.ROSRS_URI)
        self.assertEqual(config["rosrs_access_token"],  ro_test_config.ROSRS_ACCESS_TOKEN)
        self.assertEqual(config["username"],  ro_test_config.ROBOXUSERNAME)
        self.assertEqual(config["useremail"], ro_test_config.ROBOXEMAIL)
        return

    def testCreate(self):
        """
        Create a new Research Object.

        ro create RO-name [ -d dir ] [ -i RO-ident ]
        """
        rodir = self.createRoFixture(testbase, "data/ro-test-1", ro_test_config.ROBASEDIR, "ro-testCreate")
        args = [
            "ro", "create", "Test Create RO",
            "-v", 
            "-d", rodir,
            "-i", "RO-id-testCreate",
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        self.assertEqual(self.outstr.getvalue().count("ro create"), 1)
        self.checkRoFixtureManifest(rodir)
        self.checkManifestContent(rodir, "Test Create RO", "RO-id-testCreate")
        self.deleteRoFixture(rodir)
        return

    def testCreateDefaults(self):
        """
        Create a new Research Object with default options

        ro create RO-name [ -d dir ] [ -i RO-ident ]
        """
        rodir = self.createRoFixture(testbase, "data/ro-test-1", ro_test_config.ROBASEDIR, "ro-testCreateDefaults")
        args = [
            "ro", "create", "Test Create RO_+_defaults",
            "-v"
            ]
        configbase = os.path.abspath(ro_test_config.CONFIGDIR)
        save_cwd = os.getcwd()
        os.chdir(rodir)
        with SwitchStdout(self.outstr):
            status = ro.runCommand(configbase, ro_test_config.ROBASEDIR, args)
        os.chdir(save_cwd)
        assert status == 0
        self.assertEqual(self.outstr.getvalue().count("ro create"), 1)
        self.checkRoFixtureManifest(rodir)
        self.deleteRoFixture(rodir)
        return

    def testCreateBadDir(self):
        """
        Create a new Research Object with directory not in configured area

        ro create RO-name [ -d dir ] [ -i RO-ident ]
        """
        rodir = self.createRoFixture(testbase, "data/ro-test-1", ro_test_config.NOBASEDIR, "ro-testCreateBadDir")
        args = [
            "ro", "create", "Test Create RO bad directory",
            "-d", rodir,
            "-v"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        self.assertTrue(status == 1, "Expected failure due to bad RO directory");
        self.assertEqual(self.outstr.getvalue().count("research object not"), 1)
        manifestdir = rodir+"/"+ro_test_config.ROMANIFESTDIR
        self.assertFalse(os.path.exists(manifestdir), msg="checking created RO manifest dir")
        self.deleteRoFixture(rodir)
        return

    def testStatus(self):
        """
        Display status of created RO

        ro status -d rodir
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test status", "ro-testRoStatus")
        args = [
            "ro", "status",
            "-d", rodir,
            "-v"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        self.assertEqual(outtxt.count("ro status"), 1)
        self.assertRegexpMatches(outtxt, "identifier.*ro-testRoStatus")
        self.assertRegexpMatches(outtxt, "title.*RO test status")
        self.assertRegexpMatches(outtxt, "path.*%s"%rodir)
        self.assertRegexpMatches(outtxt, "creator.*%s"%ro_test_config.ROBOXUSERNAME)
        self.assertRegexpMatches(outtxt, "created")
        self.deleteTestRo(rodir)
        return

    def testStatusDefault(self):
        """
        Display status of created RO

        ro status 
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test status", "ro-testRoStatus")
        args = [
            "ro", "status",
            "-v"
            ]
        configbase = os.path.abspath(ro_test_config.CONFIGDIR)
        save_cwd = os.getcwd()
        os.chdir(rodir+"/subdir1/")
        with SwitchStdout(self.outstr):
            status = ro.runCommand(configbase, ro_test_config.ROBASEDIR, args)
        os.chdir(save_cwd)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        self.assertEqual(outtxt.count("ro status"), 1)
        self.assertRegexpMatches(outtxt, "identifier.*ro-testRoStatus")
        self.assertRegexpMatches(outtxt, "title.*RO test status")
        self.assertRegexpMatches(outtxt, "path.*%s"%rodir)
        self.assertRegexpMatches(outtxt, "creator.*%s"%ro_test_config.ROBOXUSERNAME)
        self.assertRegexpMatches(outtxt, "created")
        self.deleteTestRo(rodir)
        return

    def testList(self):
        """
        Display contents of created RO

        ro ls -d rodir
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test list", "ro-testRoList")
        args = [
            "ro", "ls",
            "-d", rodir,
            "-v"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        self.assertEqual(outtxt.count("ro list"), 1)
        self.assertRegexpMatches(outtxt, "README-ro-test-1")
        self.assertRegexpMatches(outtxt, "subdir1/subdir1-file.txt")
        self.assertRegexpMatches(outtxt, "subdir2/subdir2-file.txt")
        self.deleteTestRo(rodir)
        return

    def testListDefault(self):
        """
        Display contents of created RO containing current directory

        ro ls
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test list", "ro-testRoList")
        args = [
            "ro", "ls",
            "-v"
            ]
        configbase = os.path.abspath(ro_test_config.CONFIGDIR)
        save_cwd = os.getcwd()
        os.chdir(rodir+"/subdir2/")
        with SwitchStdout(self.outstr):
            status = ro.runCommand(configbase, ro_test_config.ROBASEDIR, args)
        os.chdir(save_cwd)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        self.assertEqual(outtxt.count("ro list"), 1)
        self.assertRegexpMatches(outtxt, "README-ro-test-1")
        self.assertRegexpMatches(outtxt, "subdir1/subdir1-file.txt")
        self.assertRegexpMatches(outtxt, "subdir2/subdir2-file.txt")
        self.deleteTestRo(rodir)
        return

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            #, "testHelpVersion"
            #, "testHelpOptions"
            , "testHelpCommands"
            , "testInvalidCommand"
            , "testConfig"
            , "testConfigVerbose"
            , "testCreate"
            , "testCreateDefaults"
            , "testCreateBadDir"
            , "testStatus"
            , "testStatusDefault"
            , "testList"
            , "testListDefault"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestBasicCommands, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestBasicCommands.log", getTestSuite, sys.argv)

# End.



# Copyright 2007 Google Inc.

import tpg
import re
import copy
import unicodedata

# Each sub_XXX function takes the following arguments:
# variables - A dict of variable names to default values.
# arg - the template <arg> string
# values - Values for variables supplied by the caller.

def sub_identity(variables, arg, values, varorder):
    key = variables.keys()[0]
    return values.get(key, variables[key] or "")

def sub_prefix(variables, arg, values, varorder):
    key = variables.keys()[0]
    value = values.get(key, variables[key])
    if None == value:
        return ""
    elif type(value) == type([]):
        return arg + arg.join(value)
    else:
        return arg + value                        


def sub_postfix(variables, arg, values, varorder):
    key = variables.keys()[0]
    value = values.get(key, variables[key])
    if None == value:
        return ""
    elif type(value) == type([]):
        return arg.join(value) + arg
    else:
        return value + arg               


# Is order important?
def sub_join(variables, arg, values, varorder):
  retval = []
  for key in varorder:
    value = values.get(key, variables[key])
    if None != value:
      retval.append("%s=%s" % (key, value))
  return arg.join(retval)

def sub_listjoin(variables, arg, values, varorder):
  key = variables.keys()[0]
  value = values.get(key, [])
  return arg.join(value)

def sub_if_non_zero(variables, arg, values, varorder):
  for key in variables.keys():
    value = values.get(key, None)
    if None != value:
      if (getattr(value, '__iter__', False) and len(value) == 0):
        continue 
      else:
        return arg
  return ""

def sub_if_zero(variables, arg, values, varorder):
  if sub_if_non_zero(variables, arg, values, varorder):
    return ""
  else:
    return arg

def unreserved(c):
  return (c >= "a" and c <= "z") or (c >= "A" and c <= "Z") or (c >= "0" and c <= "9") or (c in "-_~.")

def encode_unreserved(c):
  if unreserved(c):
    return c
  else:
    return "%%%02X" % ord(c)

def percent_encode_str(s):
  if None == s:
    return None
  if isinstance(s, unicode):
    s = unicodedata.normalize('NFKC', s)
    s = s.encode("utf-8")
  return "".join([encode_unreserved(c) for c in s])

def percent_encode(values):
  retval = {}
  for key, value in values.iteritems():
    if (getattr(value, '__iter__', False)):
      retval[key] = [percent_encode_str(s) for s in value]
    else:
      retval[key] = percent_encode_str(value)
  return retval

class ParsedTemplate(object):
  def __init__(self, vars, substitute, arg, varorder):
    self.vars = vars
    self.substitute = substitute
    self.arg = arg
    self.varorder = varorder

  def sub(self, values):
    return self.substitute(self.vars, self.arg, percent_encode(values), self.varorder)

  def variables(self):
    return self.vars.keys()

# The result of parsing is a ParsedTemplate
class Parser(tpg.Parser):
  r"""
  set lexer = ContextSensitiveLexer

  token arg        '[^\|]*';
  token varname    '[a-zA-Z0-9][a-zA-Z0-9\_\.\-]*' ;
  token vardefault '[\w\-\.\~\%]+' ;
  token num        '[0-9]+';

  START/e    -> Template/e;
  Template/t ->
       IdentityOperatorTemplate/tpl                       $ t = ParsedTemplate(*tpl)
     | OperatorTemplate/tpl                               $ t = ParsedTemplate(*tpl)
     ;
  IdentityOperatorTemplate/i -> Var/var                   $ i = (var[0], sub_identity, "", var[1])$ ;
  OperatorTemplate/o         ->
       '-suffix\|'   arg/arg '\|' Var/var                 $ o = (var[0], sub_postfix,     arg, var[1])
     | '-prefix\|'   arg/arg '\|' Var/var                 $ o = (var[0], sub_prefix,      arg, var[1])
     | '-join\|'     arg/arg '\|' Vars/var                $ o = (var[0], sub_join,        arg, var[1])
     | '-list\|'     arg/arg '\|' VarNoDefault/var        $ o = (var[0], sub_listjoin,    arg, var[1])
     | '-opt\|'      arg/arg '\|' Vars/var                $ o = (var[0], sub_if_non_zero, arg, var[1])
     | '-neg\|'      arg/arg '\|' Vars/var                $ o = (var[0], sub_if_zero,     arg, var[1])
     ;
  Vars/var                     -> Var/var
                                 (
                                    ',' Var/i             $ var = (dict(var[0].items() + i[0].items()), var[1] + i[1]) $
                                 ) * ;
  Var/var                      -> varname/name            $ var = ({name: None}, [name])
                                 (
                                   '=' vardefault/default $ var[0][name] = default
                                 ) ? ;
  VarNoDefault/var             -> varname/name            $ var = ({name: None}, [name])$ ;
  """


class DummyParsed(object):
  def __init__(self, s):
    self.s = s

  def sub(self, values):
    return self.s

  def variables(self):
    return {}


class URITemplate(object):
  def __init__(self, uri_template):
    self.uri_template = uri_template
    parts = re.split("(\{[^\}]*\})", uri_template)
    self.parsed_templates = []
    parser = Parser()
    for p in parts:
      if p and p[0] == '{':
        self.parsed_templates.append(parser(p[1:-1]))
      else:
        self.parsed_templates.append(DummyParsed(p))

  def variables(self):
    retval = set() 
    for p in self.parsed_templates:
      retval.update(set(p.variables()))
    return retval

  def sub(self, values):
    retval = []
    for p in self.parsed_templates:
      retval.append(p.sub(values))
    return "".join(retval)


if __name__ == "__main__":
  import unittest

  class Test(unittest.TestCase):
    def test_syntax_errors(self):
      p = Parser()
      syntax_errors = [
        "fred=",
        "f:",
        "f<",
        "<:",
        "<:fred,barney",
        ">:",
        ">:fred,barney"
      ]
      for s in syntax_errors:
        try:
          p(s)
          self.fail("Syntax error should have been raised.")
        except tpg.SyntacticError:
          pass

    def test_pre(self):
      p = Parser()
      test_cases = [
      # ( template,  values,  expected)
      ( "foo",       {},                 "" ),
      ( "foo",       {"foo": "barney"},  "barney" ),
      ( "foo=wilma", {},                 "wilma" ),
      ( "foo=wilma", {"foo": "barney"},  "barney" ),

      ( "-prefix|&|foo",       {},                 "" ),
      ( "-prefix|&|foo=wilma", {},                 "&wilma" ),
      ( "-prefix||foo=wilma", {},                 "wilma" ),
      ( "-prefix|&|foo=wilma", {"foo": "barney"},  "&barney" ),
      ( "-prefix|&|foo", {"foo": ["wilma", "barney"]},  "&wilma&barney" ),
 
      ( "-suffix|/|foo",        {},                 "" ),
      ( "-suffix|#|foo=wilma",  {},                 "wilma#" ),
      ( "-suffix|&?|foo=wilma", {"foo": "barney"},  "barney&?" ),
      ( "-suffix|&|foo", {"foo": ["wilma", "barney"]},  "wilma&barney&" ),

      ( "-join|/|foo",                  {},                 "" ),
      ( "-join|/|foo,bar",                  {},                 "" ),
      ( "-join|&|q,num",                  {},                 "" ),
      ( "-join|#|foo=wilma",            {},                 "foo=wilma"),
      ( "-join|#|foo=wilma,bar",        {},                 "foo=wilma"),
      ( "-join|#|foo=wilma,bar=barney", {},                 "foo=wilma#bar=barney"),
      ( "-join|&?|foo=wilma",           {"foo": "barney"},  "foo=barney" ),

      ( "-list|/|foo",       {},                    "" ),
      ( "-list|/|foo",       {"foo": ["a", "b"]},   "a/b"),
      ( "-list||foo",       {"foo": ["a", "b"]},   "ab"),
      ( "-list|/|foo",       {"foo": ["a"]},        "a"),
      ( "-list|/|foo",       {"foo": []},           ""),

      ( "-opt|&|foo",       {},                      ""),
      ( "-opt|&|foo",       {"foo": "fred"},         "&"),
      ( "-opt|&|foo",       {"foo": []},             ""),
      ( "-opt|&|foo",       {"foo": ["a"]},          "&"),
      ( "-opt|&|foo,bar",   {"foo": ["a"]},          "&"),
      ( "-opt|&|foo,bar",   {"bar": "a"},         "&"),
      ( "-opt|&|foo,bar",   {},                      ""),

      ( "-neg|&|foo",       {},                     "&"),
      ( "-neg|&|foo",       {"foo": "fred"},        ""),
      ( "-neg|&|foo",       {"foo": []},            "&"),
      ( "-neg|&|foo",       {"foo": ["a"]},         ""),
      ( "-neg|&|foo,bar",   {"bar": "a"},           ""),
      ( "-neg|&|foo,bar",   {"bar": []},            "&"),

      ( "foo",          {"foo": " "},           "%20" ),
      ( "-list|&|foo",       {"foo": ["&", "&", "|", "_"]},   "%26&%26&%7C&_" )

      ]

      i = 0
      for (template, values, expected) in test_cases:
        i += 1
        parsed = p(template)
        self.assertEqual(expected, parsed.sub(values), "Test case #%d [%s != %s]" % (i, expected, parsed.sub(values)))

  class TestURITemplate(unittest.TestCase):
    def test_simple(self):
      t = URITemplate("http://example.org/news/{id}/")
      self.assertEqual(set(["id"]), t.variables())
      self.assertEqual("http://example.org/news/joe/", t.sub({"id": "joe"}))

      t = URITemplate("http://www.google.com/notebook/feeds/{userID}{-prefix|/notebooks/|notebookID}{-opt|/-/|categories}{-list|/|categories}?{-join|&|updated-min,updated-max,alt,start-index,max-results,entryID,orderby}")
      self.assertEqual(set(['max-results', 'orderby', 'notebookID',
        'start-index', 'userID', 'updated-max', 'entryID', 'alt',
        'updated-min', 'categories']), t.variables())
      self.assertEqual("http://www.google.com/notebook/feeds/joe?", t.sub({"userID": "joe"}))
      self.assertEqual("http://www.google.com/notebook/feeds/joe/-/A%7C-B/-C?start-index=10",
          t.sub({"userID": "joe", "categories": ["A|-B", "-C"], "start-index":
            "10"}))

  unittest.main()



# ro_utils.py

"""
Research Object management supporting utility functions
"""

import os.path
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json
import re
import logging

log = logging.getLogger(__name__)

CONFIGFILE = ".ro_config"

def ronametoident(name):
    """
    Turn resource object name into an identifier containing only letters, digits and underscore characters
    """
    name = re.sub(r"\s", '_', name)         # spaces, etc. -> underscores
    name = re.sub(r"\W", "", name)          # Non-identifier characters -> remove
    return name

def progname(args):
    return os.path.basename(args[0])

def ropath(ro_config, dir):
    rodir  = os.path.abspath(dir)
    robase = os.path.realpath(ro_config['robase'])
    log.debug("ropath: rodir  %s"%(rodir))
    log.debug("ropath: robase %s"%(robase))
    if os.path.isdir(rodir) and os.path.commonprefix([robase, os.path.realpath(rodir)]) == robase:
       return rodir
    return None

def configfilename(configbase):
    return os.path.abspath(configbase+"/"+CONFIGFILE)

def writeconfig(configbase, config):
    """
    Write supplied configuration dictionary to indicated directory
    """
    configfile = open(configfilename(configbase), 'w')
    json.dump(config, configfile, indent=4)
    configfile.write("\n")
    configfile.close()
    return

def resetconfig(configbase):
    """
    Reset configuration in indicated directory
    """
    ro_config = {
        "robase":               None,
        "rosrs_uri":            None,
        "rosrs_access_token":   None,
        "username":             None,
        "useremail":            None,
        "annotationTypes":      None,
        "annotationPrefixes":   None,
        }
    writeconfig(configbase, ro_config)
    return

def readconfig(configbase):
    """
    Read configuration in indicated directory and return as a dictionary
    """
    ro_config = {
        "robase":               None,
        "rosrs_uri":            None,
        "rosrs_access_token":   None,
        "username":             None,
        "useremail":            None,
        "annotationTypes":      None,
        "annotationPrefixes":   None,
        }
    configfile = None
    try:
        configfile = open(configfilename(configbase), 'r')
        ro_config  = json.load(configfile)
    finally:
        if configfile: configfile.close()
    return ro_config

# End.
